/*
 * common.c
 *
 * Created: 2019/04/25 19:46:46
 *  Author: mNEFO
 */ 

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "extern.h"
#include "uart.h"
#include "xitoa.h"

// wait utility
void wait_ms(uint16_t t)
{
	while(t--){_delay_ms(1);}
}

void wait_us(uint16_t t)
{
	while(t--){_delay_us(1);}
}

// GPS LED(blue)
void on_gps_led(void)
{
	GPS_LED_PORT |= _BV(GPS_LED);
}

void off_gps_led(void)
{
	GPS_LED_PORT &= (uint8_t)~_BV(GPS_LED);
}

void toggle_gps_led(void)
{
	GPS_LED_PORT ^= _BV(GPS_LED);
}

// GPS control
void start_gps(void)
{
	GPS_PS_PORT |= _BV(GPS_PS);
	global_gps_count = GPS_COUNT;

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK101*32\n")); // hot start
	xfunc_out = uart1_put; // // change xitoa function to uart1_put
}

void stop_gps(void)
{
	//GPS_PS_PORT &= (uint8_t)~_BV(GPS_PS);
	global_gps_count = GPS_COUNT;

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK161,0*28\n")); // standby
	xfunc_out = uart1_put; // // change xitoa function to uart1_put
}

void reset_gps(void)
{
	GPS_PS_PORT |= _BV(GPS_PS);
	global_gps_count = GPS_COUNT;

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK104*37\n")); // full cold start(factory reset)
	xfunc_out = uart1_put; // change xitoa function to uart1_put
}

// HVPS control
// pulldown is on
void on_hvps(void)
{
	HVPS_SW_PORT &= (uint8_t)~_BV(HVPS_SW);
}

// pullup is off
void off_hvps(void)
{
	HVPS_SW_PORT |= _BV(HVPS_SW);
}

// Initialize duty
void init_duty(void)
{
	uint8_t i;

	for(i=0; i<DIGIT_NUM; i++)
	{
		global_digit_duty[i] = DUTY_NORMAL;
		global_digit_UP_dot_pwm[i] = 0;
		global_digit_DP_dot_pwm[i] = 0;
	}
}

// Normalize duty
void normal_duty(void)
{
	uint8_t i;

	for(i=0; i<DIGIT_NUM; i++)
	{
		global_digit_duty[i] = global_config_data.duty;
	}
}

// Minimize duty
void minimum_duty(void)
{
	uint8_t i;

	// set minimum temporarily
	for(i=0; i<DIGIT_NUM; i++)
	{
		global_digit_duty[i] = DUTY_MIN;
	}
}

// Show operation time
void show_op_time(void)
{
	xprintf(PSTR("OPERATION TIME=%d [sec] \n"), global_config_data.sec_count);
}

// Reset all setting & show zero.
void reset_all_setting(void)
{
	uint8_t i;

	// stop WDT
	wdt_disable();

	reset_gps();

	// stop interrupt
	cli();

	reset_eeprom_setting();
	reset_DS3234S();

	// show zero loop
	while(1)
	{
		for(i=0; i<DIGIT_NUM; i++)
		{
			if(i < NIXIE_ANODE_DVIDE) // PORTA 0-7
			{
				// turn on anode, output '0' cathode, on dp and up
				NIXIE_ANODE_PORT_0 |= _BV(i);
				NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_8;
				NIXIE_CATHODE_UP_DOT = DOT_PWM_MAX;
				NIXIE_CATHODE_DP_DOT = DOT_PWM_MAX;
				wait_us(300);

				// turn off anode, output '0' cathode, on dp and up
				NIXIE_ANODE_PORT_0 &= (uint8_t)~_BV(i);
				NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_N;
				NIXIE_CATHODE_UP_DOT = 0;
				NIXIE_CATHODE_DP_DOT = 0;
				wait_us(50);
			}
			else // PORTF 0-3
			{
				// turn on anode, output '0' cathode, on dp and up
				NIXIE_ANODE_PORT_1 |= _BV(i - NIXIE_ANODE_DVIDE);
				NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_8;
				NIXIE_CATHODE_UP_DOT = DOT_PWM_MAX;
				NIXIE_CATHODE_DP_DOT = DOT_PWM_MAX;
				wait_us(300);

				// turn off anode, output '0' cathode, on dp and up
				NIXIE_ANODE_PORT_1 &= (uint8_t)~_BV(i - NIXIE_ANODE_DVIDE);
				NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_N;
				NIXIE_CATHODE_UP_DOT = 0;
				NIXIE_CATHODE_DP_DOT = 0;
				wait_us(50);				
			}
		}
	}
}

// xor shift random
// 0 to 2^8 - 1
uint8_t rand_xor_shift(void)
{
	static uint8_t rand = XOR_SHIFT_SEED;
	static uint8_t count = 0;
	static uint8_t num = 0;
	
	rand ^= rand >> global_xor_shift_const[num][0];
	rand ^= rand << global_xor_shift_const[num][1];
	rand ^= rand >> global_xor_shift_const[num][2];

	// change const
	count++;
	if(count > 250){num++;}
	if(num == 24)
	{
		num = 0;
	}

	return rand;
}

// leap year
uint8_t check_leap_year(uint8_t century, uint8_t year)
{
	uint16_t all_year;

	all_year = (uint16_t)century * 100 + (uint16_t)year;

	if(((all_year % 4) == 0 && (all_year % 100) != 0) || (all_year % 400) == 0)
	{
		return 29;
	}
	return 28;
}

// month end
uint8_t get_monthend(uint8_t century, uint8_t year, uint8_t month)
{
	switch(month)
	{
		case 1:
		return 31;

		case 2:
		return check_leap_year(century, year);

		case 3:
		return 31;

		case 4:
		return 30;

		case 5:
		return 31;

		case 6:
		return 30;

		case 7:
		return 31;

		case 8:
		return 31;

		case 9:
		return 30;

		case 10:
		return 31;

		case 11:
		return 30;

		case 12:
		return 31;

		default:
		return 31;
	}

	return 31;
}

// Zeller's congruence
uint8_t get_zeller_day(uint8_t century, uint8_t year, uint8_t month, uint8_t date)
{
	uint16_t day = 0;
	uint16_t all_year = 0;
	
	all_year  = (uint16_t)century * 100 + (uint16_t)year;
	if(month < 3)
	{
		all_year--;
		month+=12;
	}
	day = (all_year + (uint16_t)(all_year / 4) - (uint16_t)(all_year / 100) + (uint16_t)(all_year / 400) + (uint16_t)((13 * (uint16_t)month + 8) / 5) + (uint16_t)date) % 7;
	// this formula gives 0-6, ds3232s needs 1-7
	day++;

	return day;
}

