/*
 * eeprom.c
 *
 * Created: 2019/04/25 9:57:41
 *  Author: mNEFO
 */ 

#include <avr/eeprom.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>

#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"

// write all data to eeprom
void eep_write_config(void)
{
	eeprom_busy_wait();
	eeprom_write_block(&global_config_data, &global_config_eep_map, sizeof(CONFIG_DATA));
}

// read all data from eeprom
void eep_read_config(void)
{
	eeprom_busy_wait();
	eeprom_read_block(&global_config_data, &global_config_eep_map, sizeof(CONFIG_DATA));
}

// read setting & check eeprom data
void read_eeprom_setting(void)
{
	uint8_t temp;

	// read config
	eep_read_config();

	// check config data
	// mode
	if(global_config_data.mode < 0 || global_config_data.mode > MODE_NORMAL_MDTS)
	{
		reset_eeprom_setting();
		return;
	}

	// century upper 4bit is 10 century, lower 4bit is 1 century
	temp = 0b00001111 & global_config_data.century;
	if(temp > 9)
	{
		reset_eeprom_setting();
		return;
	}
	temp = (0b11110000 & global_config_data.century) >> 4;
	if(temp > 9)
	{
		reset_eeprom_setting();
		return;
	}

	// duty
	if(global_config_data.duty < DUTY_MIN || global_config_data.duty > DUTY_MAX)
	{
		reset_eeprom_setting();
		return;
	}

	// dot mode
	if(global_config_data.dot_mode < 0 || global_config_data.dot_mode >= DOT_END)
	{
		reset_eeprom_setting();
		return;
	}

	// sec count

	// use light sensor
	if(global_config_data.use_light_sensor < DISABLE || global_config_data.use_light_sensor > ENABLE)
	{
		reset_eeprom_setting();
		return;
	}

	// light sensor threshold
	if(global_config_data.light_sensor_th < LIGHT_SENSOR_TH_MIN || global_config_data.light_sensor_th > LIGHT_SENSOR_TH_MAX)
	{
		reset_eeprom_setting();
		return;
	}

	// light sensor hvps
	if(global_config_data.light_sensor_hvps < DISABLE || global_config_data.light_sensor_hvps > ENABLE)
	{
		reset_eeprom_setting();
		return;
	}

	// use gps
	if(global_config_data.use_gps < DISABLE || global_config_data.use_gps > ENABLE)
	{
		reset_eeprom_setting();
		return;
	}

	// gps log
	if(global_config_data.gps_log < DISABLE || global_config_data.gps_log > ENABLE)
	{
		reset_eeprom_setting();
		return;
	}

	// gps timezone
	if(global_config_data.gps_tz < GPS_TIMEZONE_MIN || global_config_data.gps_tz > GPS_TIMEZONE_MAX)
	{
		reset_eeprom_setting();
		return;
	}
}

// reset eeprom setting & write default
void reset_eeprom_setting(void)
{
	uint8_t century[2];

	century[0] = E_CENTURY_DEFAULT / 10;
	century[1] = E_CENTURY_DEFAULT - century[0] * 10;

	// setting
	global_config_data.mode = E_MODE_DEFAULT;
	global_config_data.century = century[0] << 4 | century[1];
	global_config_data.duty = E_NIXIE_DUTY_DEFAULT;
	global_config_data.dot_mode = E_DOT_MODE_DEFAULT;
	global_config_data.sec_count = E_SEC_COUNT_DEFAULT;
	global_config_data.use_light_sensor = E_USE_LIGHT_SENSOR_DEFAULT;
	global_config_data.light_sensor_th = E_LIGHT_SENSOR_TH_DEFAULT;
	global_config_data.light_sensor_hvps = E_LIGHT_SENSOR_HVPS_DEFAULT;
	global_config_data.use_gps = E_USE_GPS_DEFAULT;
	global_config_data.gps_log = E_USE_GPS_LOG_DEFAULT;
	global_config_data.gps_tz = E_GPS_TIMEZONE_DEFAULT;
	eep_write_config();
}

void eep_update_mode(uint8_t mode)
{
	if(mode >= MODE_NORMAL_YMDT && mode <= MODE_NORMAL_MDTS)
	{
		global_config_data.mode = mode;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.mode, mode);
	}
}

void eep_update_century(uint8_t century)
{
	uint8_t temp[2];
	
	if(century > 99){return;}
	sprintf((char *)temp, "%02d", century);
	CHARNUM_TO_NUM(temp[0]);
	CHARNUM_TO_NUM(temp[1]);
	global_config_data.century = temp[0] << 4 | temp[1];
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.century, global_config_data.century);
}

void eep_update_duty(uint8_t duty)
{
	uint8_t i;

	if(duty >= DUTY_MIN && duty <= DUTY_MAX)
	{
		global_config_data.duty = duty;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.duty, duty);
		for(i=0; i<DIGIT_NUM; i++)
		{
			global_digit_duty[i] = duty;
		}
	}
}

void eep_toggle_dot_mode(void)
{
	global_config_data.dot_mode++;
	if(global_config_data.dot_mode == DOT_END){global_config_data.dot_mode = DOT_PWM;}
	eep_update_dot_mode(global_config_data.dot_mode);
}

void eep_update_dot_mode(uint8_t dot_mode)
{
	if(dot_mode >= 0 && dot_mode < DOT_END)
	{
		global_config_data.dot_mode = dot_mode;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.dot_mode, dot_mode);
	}
}

void eep_update_sec_count(uint32_t sec_count)
{
	global_config_data.sec_count = sec_count;
	eeprom_busy_wait();
	eeprom_update_dword((uint32_t *)&global_config_eep_map.sec_count, sec_count);
}

void eep_update_use_light_sensor(uint8_t use_light_sensor)
{
	if(use_light_sensor >= DISABLE && use_light_sensor <= ENABLE)
	{
		global_config_data.use_light_sensor = use_light_sensor;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_light_sensor, use_light_sensor);
	}
	normal_duty();
}

void eep_update_light_sensor_th(uint16_t light_sensor_th)
{
	if(light_sensor_th >= LIGHT_SENSOR_TH_MIN && light_sensor_th <= LIGHT_SENSOR_TH_MAX)
	{
		global_config_data.light_sensor_th = light_sensor_th;
		eeprom_busy_wait();
		eeprom_write_word((uint16_t *)&global_config_eep_map.light_sensor_th, light_sensor_th);
	}
}

void eep_update_light_sensor_hvps(uint8_t light_sensor_hvps)
{
	if(light_sensor_hvps >= DISABLE && light_sensor_hvps <= ENABLE)
	{
		global_config_data.light_sensor_hvps = light_sensor_hvps;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.light_sensor_hvps, light_sensor_hvps);
	}
}

void eep_update_use_gps(uint8_t use_gps)
{
	if(use_gps >= DISABLE && global_config_data.use_gps <= ENABLE)
	{
		global_config_data.use_gps = use_gps;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_gps, use_gps);
	}
}

void eep_update_gps_log(uint8_t gps_log)
{
	if(gps_log >= DISABLE && gps_log <= ENABLE)
	{
		global_config_data.gps_log = gps_log;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.gps_log, gps_log);
	}
}

void eep_update_gps_tz(int8_t gps_tz)
{
	if(gps_tz >= GPS_TIMEZONE_MIN && gps_tz <= GPS_TIMEZONE_MAX)
	{
		global_config_data.gps_tz = gps_tz;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.gps_tz, gps_tz);
	}
}

void eep_update_serial(uint32_t serial)
{
	global_config_data.serial = serial;
	eeprom_busy_wait();
	eeprom_update_dword((uint32_t *)&global_config_eep_map.serial, serial);
}