/*
 * uart.c
 *
 * Created: 2019/04/25 19:59:21
 *  Author: mNEFO
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "common.h"
#include "extern.h"
#include "xitoa.h"
#include "uart.h"

typedef struct _uart_fifo
{
	volatile uint8_t wi;
	volatile uint8_t ri;
	volatile uint8_t ct;
	volatile uint8_t buf[UART_BUF];
} UART_FIFO;

// uart0 is GPS
volatile UART_FIFO global_TxFifo0 = {0}, global_RxFifo0 = {0};

// uart1 is FT230XS
volatile UART_FIFO global_TxFifo1 = {0}, global_RxFifo1 = {0};

void init_uart(void)
{
	uint8_t i;

	// zero fill
	global_TxFifo0.ct = 0;
	global_TxFifo0.wi = 0;
	global_TxFifo0.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_TxFifo0.buf[i] = 0;
	}
	global_RxFifo0.ct = 0;
	global_RxFifo0.wi = 0;
	global_RxFifo0.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_RxFifo0.buf[i] = 0;
	}
	global_TxFifo1.ct = 0;
	global_TxFifo1.wi = 0;
	global_TxFifo1.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_TxFifo1.buf[i] = 0;
	}
	global_RxFifo1.ct = 0;
	global_RxFifo1.wi = 0;
	global_RxFifo1.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_RxFifo1.buf[i] = 0;
	}

	// UBRR0n = F_CPU / 16 / BAUD  - 1;
	// uart0 gps
	UBRR0H = 0;
	UBRR0L = 103; // 9600 bps @ 16MHz
	UCSR0B |= _BV(RXCIE0) | _BV(RXEN0)| _BV(TXEN0);

	// uart1 FT230XS
	UBRR1H = 0;
	UBRR1L = 103; // 9600 bps @ 16MHz
	UCSR1B |= _BV(RXCIE1) | _BV(RXEN1)| _BV(TXEN1);
}

void uart0_put(uint8_t data)
{
	uint8_t i;

	i = global_TxFifo0.wi;
	// prevent buffer overflow
	while(global_TxFifo0.ct >= sizeof(global_TxFifo0.buf));
	global_TxFifo0.buf[i++] = data;
	global_TxFifo0.ct++;
	// enable UDRIE0
	UCSR0B |= _BV(UDRIE0);
	global_TxFifo0.wi = i % sizeof(global_TxFifo0.buf);
}

uint8_t uart0_get(void)
{
	uint8_t i;
	uint8_t d;

	i = global_RxFifo0.ct;
	d = 0;
	if(i > 0)
	{
		i = global_RxFifo0.ri;
		d = global_RxFifo0.buf[i++];
		global_RxFifo0.ri = i % sizeof(global_RxFifo0.buf);
		global_RxFifo0.ct--;
	}

	return d;
}

void uart0_tx_flush(void)
{
	uint8_t i;

	global_TxFifo0.ct = 0;
	global_TxFifo0.wi = 0;
	global_TxFifo0.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_TxFifo0.buf[i] = 0;
	}
}

void uart0_rx_flush(void)
{
	uint8_t i;

	global_RxFifo0.ct = 0;
	global_RxFifo0.wi = 0;
	global_RxFifo0.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_RxFifo0.buf[i] = 0;
	}
}

// GPS time is contained in '$GPRMC'.
// When GPS(PA6C) is 3D fixed the 17th char is 'A'.
void uart0_get_GPRMC(uint8_t *buf)
{
	// $GPRMC & 'A' flag
	//if(buf[0] == '$' && buf[1] == 'G' && buf[2] == 'P' && buf[3] == 'R' && buf[4] == 'M' && buf[5] == 'C')
	if(buf[5] == 'C' && buf[18] == 'A')
	{
		global_gpstime.time[0] = buf[7];
		global_gpstime.time[1] = buf[8];
		global_gpstime.time[2] = buf[9];
		global_gpstime.time[3] = buf[10];
		global_gpstime.time[4] = buf[11];
		global_gpstime.time[5] = buf[12];
		global_gpstime.valid = TRUE;
	}
}

void uart0_get_line(void)
{
	uint8_t buf[LINE_BUF] = {0};
	uint8_t buf_size = sizeof(buf);
	uint8_t d;
	uint8_t i = 0;
	
	for(;;)
	{
		d = uart0_get();
		if(!d){continue;}
		if (i == 0 && d != '$') continue;
		if(d == '\n')
		{
			buf[i] = 0;
			break;
		}
		else
		{
			buf[i++] = d;
		}
		// buffer overflow
		if(i >= buf_size)
		{
			i = 0;
			continue;
		}
	}

	uart0_get_GPRMC(buf);
}

// Receive data from GPS.
ISR(USART0_RX_vect)
{
	uint8_t d;
	uint8_t i;

	d = UDR0;

	// GPS log raw output to uart1
	if(global_config_data.gps_log == TRUE)
	{
		loop_until_bit_is_set(UCSR1A, UDRE1);
		UDR1 = d;
	}

	// Receive char until '\n'
	i = global_RxFifo0.ct;
	if(i < sizeof(global_RxFifo0.buf))
	{
		// ASCII or return
		if((d >= 0x20 && d <= 0x7E) || d == '\n')
		{
			global_RxFifo0.ct = i + 1;
			i = global_RxFifo0.wi;
			global_RxFifo0.buf[i++] = d;
			// when reach the end of buffer, return to zero automatically
			global_RxFifo0.wi = i % sizeof(global_RxFifo0.buf);
		}
		// Analyze line
		if(d == '\n')
		{
			uart0_get_line();
		}
	}
	// Something happened
	else
	{
		uart0_rx_flush();
	}
}

// Send data to GPS
ISR(USART0_UDRE_vect)
{
	uint8_t i, j;

	i = global_TxFifo0.ct;
	if(i > 0)
	{
		global_TxFifo0.ct = --i;
		j = global_TxFifo0.ri;
		UDR0 = global_TxFifo0.buf[j++];
		global_TxFifo0.ri = j % sizeof(global_TxFifo0.buf);
	}
	if(i == 0)
	{
		// if no data to transmit, disable UDRIE0
		UCSR0B &= (uint8_t)~_BV(UDRIE0);
	}
}

void uart1_put(uint8_t data)
{
	uint8_t i;

	i = global_TxFifo1.wi;
	// prevent buffer overflow
	while(global_TxFifo1.ct >= sizeof(global_TxFifo1.buf));
	global_TxFifo1.buf[i++] = data;
	global_TxFifo1.ct++;
	// enable UDRIE1
	UCSR1B |= _BV(UDRIE1);
	global_TxFifo1.wi = i % sizeof(global_TxFifo1.buf);
}

void uart1_putn(uint8_t data)
{
	uint8_t i;

	i = global_TxFifo1.wi;
	// prevent buffer overflow
	while(global_TxFifo1.ct >= sizeof(global_TxFifo1.buf));
	global_TxFifo1.buf[i++] = data;
	global_TxFifo1.ct++;
	// enable UDRIE1
	UCSR1B |= _BV(UDRIE1);
	global_TxFifo1.wi = i % sizeof(global_TxFifo1.buf);

	// \r
	i = global_TxFifo1.wi;
	// prevent buffer overflow
	while(global_TxFifo1.ct >= sizeof(global_TxFifo1.buf));
	global_TxFifo1.buf[i++] = '\r';
	global_TxFifo1.ct++;
	// enable UDRIE1
	UCSR1B |= _BV(UDRIE1);
	global_TxFifo1.wi = i % sizeof(global_TxFifo1.buf);

	// \n
	i = global_TxFifo1.wi;
	// prevent buffer overflow
	while(global_TxFifo1.ct >= sizeof(global_TxFifo1.buf));
	global_TxFifo1.buf[i++] = '\n';
	global_TxFifo1.ct++;
	// enable UDRIE1
	UCSR1B |= _BV(UDRIE1);
	global_TxFifo1.wi = i % sizeof(global_TxFifo1.buf);
}

uint8_t uart1_get(void)
{
	uint8_t i;
	uint8_t d;

	i = global_RxFifo1.ct;
	d = 0;
	if(i > 0)
	{
		i = global_RxFifo1.ri;
		d = global_RxFifo1.buf[i++];
		global_RxFifo1.ri = i % sizeof(global_RxFifo1.buf);
		global_RxFifo1.ct--;
	}

	return d;
}

void uart1_tx_flush(void)
{
	uint8_t i;

	global_TxFifo1.ct = 0;
	global_TxFifo1.wi = 0;
	global_TxFifo1.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_TxFifo1.buf[i] = 0;
	}
}

void uart1_rx_flush(void)
{
	uint8_t i;

	global_RxFifo1.ct = 0;
	global_RxFifo1.wi = 0;
	global_RxFifo1.ri = 0;
	for(i=0; i<UART_BUF; i++)
	{
		global_RxFifo1.buf[i] = 0;
	}
}

void uart1_get_line(void)
{
	uint8_t buf[LINE_BUF] = {0};
	uint8_t buf_size = sizeof(buf);
	uint8_t d;
	uint8_t i = 0;
	
	for(;;)
	{
		d = uart1_get();
		if(!d){continue;}
		if(d == '\n')
		{
			buf[i] = 0;
			break;
			}else{
			buf[i++] = d;
		}
		// buffer overflow
		if(i >= buf_size)
		{
			i = 0;
			continue;
		}
	}

	//exec_command(buf);
}

// Receive data from FT230XS.
ISR(USART1_RX_vect)
{
	uint8_t d;
	uint8_t i;

	d = UDR1;
	i = global_RxFifo1.ct;

	if(i < sizeof(global_RxFifo1.buf))
	{
		// record only ASCII, space and LF(\n)
		// ASCII
		if(d >= 0x20 && d <= 0x7E)
		{
			global_RxFifo1.ct = i + 1;
			i = global_RxFifo1.wi;
			global_RxFifo1.buf[i++] = d;
			// when reach the end of buffer, return to zero automatically
			global_RxFifo1.wi = i % sizeof(global_RxFifo1.buf);
			uart1_put(d);
		}
		else if(d == '\n') // return
		{
			global_RxFifo1.ct = i + 1;
			i = global_RxFifo1.wi;
			global_RxFifo1.buf[i++] = d;
			// when reach the end of buffer, return to zero automatically
			global_RxFifo1.wi = i % sizeof(global_RxFifo1.buf);
			uart1_put('\r');
			uart1_put('\n');
			uart1_get_line();
		
		}
		else if(d == '\b') // BS
		{
			if(global_RxFifo1.ct > 0)
			{
				global_RxFifo1.ct = i - 1;
				i = global_RxFifo1.wi;
				i--;
				// when reach the end of buffer, return to zero automatically
				global_RxFifo1.wi = i % sizeof(global_RxFifo1.buf);
				uart1_put('\b');
				uart1_put(' ');
				uart1_put('\b');
			}
		}
	}
	else // Something happened.
	{
	uart1_rx_flush();
	}
}

// Send data to FT230XS.
ISR(USART1_UDRE_vect)
{
	uint8_t i, j;

	i = global_TxFifo1.ct;
	if(i > 0)
	{
		global_TxFifo1.ct = --i;
		j = global_TxFifo1.ri;
		UDR1 = global_TxFifo1.buf[j++];
		global_TxFifo1.ri = j % sizeof(global_TxFifo1.buf);
	}
	if(i == 0)
	{
		// if no data to transmit, disable UDRIE1
		UCSR1B &= (uint8_t)~_BV(UDRIE1);
	}
}
